@@ -9,7 +9,7 @@
  * See http://mod-qos.sourceforge.net/ for further
  * details.
  *
- * Copyright (C) 2017 Pascal Buchbinder
+ * Copyright (C) 2018 Pascal Buchbinder
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -28,7 +28,7 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c,v 1.114 2017-10-02 15:44:03 pbuchbinder Exp $";
+static const char revision[] = "$Id: qslog.c 2431 2018-03-20 19:38:53Z pbuchbinder $";
 
 #include <stdio.h>
 #include <string.h>
@@ -88,6 +88,12 @@ typedef struct {
   int decVal;
 } counter_rec_t;
 
+typedef struct {
+  unsigned long long lines;
+  unsigned long long ms;
+  unsigned long long pivot[21];
+} duration_t;
+
 typedef struct {
   long request_count;
   long status_1;
@@ -146,6 +152,10 @@ typedef struct stat_rec_st {
   long long byte_count;
   long long duration_count;
   long long duration_count_ms;
+  long duration_49;
+  long duration_99;
+  long duration_499;
+  long duration_999;
   long duration_0;
   long duration_1;
   long duration_2;
@@ -160,6 +170,9 @@ typedef struct stat_rec_st {
   long average_count;
   unsigned long long averAge;
   long averAge_count;
+  unsigned long max;
+
+  duration_t total;
 
   long status_1;
   long status_2;
@@ -217,6 +230,7 @@ static regex_t m_trx_g;
 /* real time mode (default) or offline */
 static int   m_off = 0;
 static int   m_offline = 0;
+static int   m_offline_s = 0;
 static int   m_offline_data = 0;
 static char  m_date_str[MAX_LINE];
 static int   m_mem = 0;
@@ -714,10 +728,11 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
   }
   if(m_customcounter) {
     // max len: 18446744073709551615
-    sprintf(custom, "s;%llu;a;%llu;A;%llu;",
+    sprintf(custom, "s;%llu;a;%llu;A;%llu;M;%lu;",
             stat_rec->sum,
             stat_rec->average / (stat_rec->average_count == 0 ? 1 : stat_rec->average_count),
-            stat_rec->averAge / (stat_rec->averAge_count == 0 ? 1 : stat_rec->averAge_count));
+            stat_rec->averAge / (stat_rec->averAge_count == 0 ? 1 : stat_rec->averAge_count),
+            stat_rec->max);
   }
   if(main) {
     sprintf(ip, "ip;%ld;", qs_countEventT(m_ip_list));
@@ -740,17 +755,7 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
           "4xx;%ld;"
           "5xx;%ld;"
           "%s"
-          NAV";%lld;"
-          "<1s;%ld;"
-          "1s;%ld;"
-          "2s;%ld;"
-          "3s;%ld;"
-          "4s;%ld;"
-          "5s;%ld;"
-          ">5s;%ld;"
-          "%s"
-          "%s"
-          ,
+          NAV";%lld;",
           timeStr,
           main ? "" : stat_rec->id,
           stat_rec->line_count/LOG_INTERVAL,
@@ -764,7 +769,29 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
           stat_rec->status_4,
           stat_rec->status_5,
           avms,
-          stat_rec->duration_count/(stat_rec->line_count == 0 ? 1 : stat_rec->line_count),
+          stat_rec->duration_count/(stat_rec->line_count == 0 ? 1 : stat_rec->line_count));
+  if(m_avms) {
+    fprintf(f,
+            "0-49ms;%ld;"
+            "50-99ms;%ld;"
+            "100-499ms;%ld;"
+            "500-999ms;%ld;",
+            stat_rec->duration_49,
+            stat_rec->duration_99,
+            stat_rec->duration_499,
+            stat_rec->duration_999);
+  }
+  fprintf(f,
+          "<1s;%ld;"
+          "1s;%ld;"
+          "2s;%ld;"
+          "3s;%ld;"
+          "4s;%ld;"
+          "5s;%ld;"
+          ">5s;%ld;"
+          "%s"
+          "%s"
+          ,
           stat_rec->duration_0,
           stat_rec->duration_1,
           stat_rec->duration_2,
@@ -813,6 +840,7 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
   stat_rec->average_count = 0;
   stat_rec->averAge = 0;
   stat_rec->averAge_count = 0;
+  stat_rec->max = 0;
   stat_rec->status_1 = 0;
   stat_rec->status_2 = 0;
   stat_rec->status_3 = 0;
@@ -820,6 +848,10 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
   stat_rec->status_5 = 0;
   stat_rec->duration_count = 0;
   stat_rec->duration_count_ms = 0;
+  stat_rec->duration_49 = 0;
+  stat_rec->duration_99 = 0;
+  stat_rec->duration_499 = 0;
+  stat_rec->duration_999 = 0;
   stat_rec->duration_0 = 0;
   stat_rec->duration_1 = 0;
   stat_rec->duration_2 = 0;
@@ -1057,6 +1089,10 @@ static stat_rec_t *createRec(apr_pool_t *pool, const char *id, const char *patte
   rec->byte_count = 0;
   rec->duration_count = 0;
   rec->duration_count_ms = 0;
+  rec->duration_49 = 0;
+  rec->duration_99 = 0;
+  rec->duration_499 = 0;
+  rec->duration_999 = 0;
   rec->duration_0 = 0;
   rec->duration_1 = 0;
   rec->duration_2 = 0;
@@ -1071,7 +1107,17 @@ static stat_rec_t *createRec(apr_pool_t *pool, const char *id, const char *patte
   rec->average_count = 0;
   rec->averAge = 0;
   rec->averAge_count = 0;
+  rec->max = 0;
 
+  rec->total.lines = 0;
+  rec->total.ms = 0;
+  {
+    int p = 0;
+    for(p = 0; p < 21; p++) {
+      rec->total.pivot[p] = 0;
+    }
+  }
+  
   rec->status_1 = 0;
   rec->status_2 = 0;
   rec->status_3 = 0;
@@ -1381,7 +1427,7 @@ static void updateClient(apr_pool_t *pool, char *T, char *t, char *D, char *S,
 static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
                       char *s, char *a, char *A,
                       char *BI, char *B, char *R, char *I, char *U, char *Q,
-                      char *E, char *k, char *C, long tme, long tmems) {
+                      char *E, char *k, char *C, char *M, long tme, long tmems) {
   if(Q != NULL) {
     if(strchr(Q, 'V') != NULL) {
       rec->qos_V++;
@@ -1449,6 +1495,13 @@ static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
     rec->averAge += atol(A);
     rec->averAge_count++;
   }
+  if(M && M[0]) {
+    long max = atol(M);
+    if(max > rec->max) {
+      rec->max = max;
+    }
+  }
+
   if(S != NULL) {
     if(S[0] == '1') {
       rec->status_1++;
@@ -1468,6 +1521,37 @@ static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
     /* response duration */
     rec->duration_count += tme;
     rec->duration_count_ms += tmems;
+    if(m_offline_s) {
+      long min = 0;
+      long max = 50;
+      int p;
+      rec->total.lines++;
+      rec->total.ms += tmems;
+      for(p = 0; p < 20; p++) {
+        // 0ms <= time < 50ms etc.
+        if((min <= tmems) && (tmems < max)) {
+          rec->total.pivot[p]++;
+          break;
+        }
+        min = max;
+        max += 50;
+      }
+      if(tmems >= 1000) {
+        // time >= 1000ms (20x50ms)
+        rec->total.pivot[20]++;
+      }
+    }
+    if(m_avms) {
+      if(tmems < 49) {
+        rec->duration_49++;
+      } else if(50 <= tmems && tmems < 99) {
+        rec->duration_99++;
+      } else if(100 <= tmems && tmems < 499) {
+        rec->duration_499++;
+      } else if(500 <= tmems && tmems < 999) {
+        rec->duration_999++;
+      }
+    }
     if(tme < 1) {
       rec->duration_0++;
     } else if(tme == 1) {
@@ -1656,24 +1740,26 @@ static void updateStat(apr_pool_t *pool, const char *cstr, char *line) {
     stripNum(&M);
     qsNoFloat(M);
   }
+
+  /* request duration */
   tme = 0;
-  if(T != NULL || t != NULL || D != NULL) {
-    /* response duration */
-    tmems = 0;
-    if(T) {
-      stripNum(&T);
-      tme = atol(T);
-    } else if(t) {
-      stripNum(&t);
-      tmems= atol(t);
-      tme = tmems / 1000;
-    } else if(D) {
-      stripNum(&D);
-      tmems = atol(D);
-      tmems = tmems / 1000;
-      tme = tmems / 1000;
-    }
+  tmems = 0;
+  if(T) {
+    stripNum(&T);
+    tme = atol(T);
   }
+  if(t) {
+    stripNum(&t);
+    tmems= atol(t);
+    tme = tmems / 1000;
+  }
+  if(D) {
+    stripNum(&D);
+    tmems = atol(D);
+    tmems = tmems / 1000;
+    tme = tmems / 1000;
+  }
+
   if(m_offline_count) {
     updateClient(pool, T, t, D, S, BI, B, R, I, U, Q, E, k, C, M, ct, tme, tmems, m);
   } else if(m_offline_url) {
@@ -1682,9 +1768,9 @@ static void updateStat(apr_pool_t *pool, const char *cstr, char *line) {
     }
     updateUrl(pool, R, S, tmems);
   } else {
-    updateRec(m_stat_rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, E, k, C, tme, tmems);
+    updateRec(m_stat_rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, E, k, C, M, tme, tmems);
     if(rec) {
-      updateRec(rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, E, k, C, tme, tmems);
+      updateRec(rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, E, k, C, M, tme, tmems);
     }
   }
   qs_csUnLock();
@@ -2003,8 +2089,11 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "  - repsonse status codes within the last minute (1xx,2xx,3xx,4xx,5xx)\n");
   qs_man_println(man, "  - average response duration ("NAV")\n");
   qs_man_println(man, "  - average response duration in milliseconds ("NAVMS")\n");
-  qs_man_println(man, "  - distribution of response durations within the last minute\n");
-  qs_man_print(man, "    (<1s,1s,2s,3s,4s,5s,>5)\n");
+  qs_man_println(man, "  - distribution of response durations in seconds within the last minute\n");
+  qs_man_print(man, "    (<1s,1s,2s,3s,4s,5s,>5s)\n");
+  if(man) printf("\n");
+  qs_man_println(man, "  - distribution of response durations faster than a second within the last minute\n");
+  qs_man_print(man, "    (0-49ms,50-99ms,100-499ms,500-999ms)\n");
   if(man) printf("\n");
   qs_man_println(man, "  - number of established (new) connections within the measured time (esco)\n");
   qs_man_println(man, "  - average system load (sl)\n");
@@ -2037,9 +2126,9 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "     S defines HTTP response status code (%%s)\n");
   qs_man_println(man, "     B defines the transferred bytes (%%b or %%O)\n");
   qs_man_println(man, "     i defines the received bytes (%%I)\n");
-  qs_man_println(man, "     T defines the request duration (%%T)\n");
-  qs_man_println(man, "     t defines the request duration in milliseconds (may be used instead of T)\n");
-  qs_man_println(man, "     D defines the request duration in microseconds (may be used instead of T) (%%D)\n");
+  qs_man_println(man, "     D defines the request duration in microseconds (%%D)\n");
+  qs_man_println(man, "     t defines the request duration in milliseconds (may be used instead of D)\n");
+  qs_man_println(man, "     T defines the request duration in seconds (may be used instead of D or t) (%%T)\n");
   qs_man_println(man, "     k defines the number of keepalive requests on the connection (%%k)\n");
   qs_man_println(man, "     U defines the user tracking id (%%{mod_qos_user_id}e)\n");
   qs_man_println(man, "     Q defines the mod_qos_ev event message (%%{mod_qos_ev}e)\n");
@@ -2047,7 +2136,7 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "     s arbitrary counter to add up (sum within a minute)\n");
   qs_man_println(man, "     a arbitrary counter to build an average from (average per request)\n");
   qs_man_println(man, "     A arbitrary counter to build an average from (average per request)\n");
-  qs_man_println(man, "     M maximum of an ever reached value, available in -pc mode only\n");
+  qs_man_println(man, "     M arbitrary counter to measure the maximum value reached (peak)\n");
   qs_man_println(man, "     E comma separated list of event strings\n");
   qs_man_println(man, "     c content type (%%{content-type}o), available in -pc mode only\n");
   qs_man_println(man, "     m request method (GET/POST) (%%m), available in -pc mode only\n");
@@ -2295,6 +2384,10 @@ int main(int argc, const char *const argv[]) {
     } else if(strcmp(*argv,"-p") == 0) {   /* activate offline analysis */
       m_offline = 1;
       qs_set2OfflineMode();
+    } else if(strcmp(*argv,"-ps") == 0) {   /* activate offline analysis (inckl. summary) */
+      m_offline = 1;
+      m_offline_s = 1;
+      qs_set2OfflineMode();
     } else if(strcmp(*argv,"-pc") == 0) {  /* activate offline counting analysis */
       m_offline_count = 1;
       qs_set2OfflineMode();
@@ -2560,6 +2653,20 @@ int main(int argc, const char *const argv[]) {
     if(!m_verbose) {
       fprintf(stdout, "\n");
     }
+    if(m_offline_s) {
+      int p;
+      int min = 0;
+      int max = 50;
+      printf("\n");
+      printf("      requests: %llu\n", m_stat_rec->total.lines);
+      printf("       average: %llums\n", m_stat_rec->total.ms/m_stat_rec->total.lines);
+      for(p = 0; p <20; p++) {
+        printf("%3dms - %4dms: %lld\n", min, max, m_stat_rec->total.pivot[p]);
+        min = max;
+        max += 50;
+      }
+      printf("1000ms+       : %lld\n", m_stat_rec->total.pivot[20]);
+    }
   } else {
     /* standard mode reads data from
      * stdin and uses a separate thread
